#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
====
Frontal Horus
====

description
-----------

Interface graphique de gestion de l'annuaire LDAP d'horus (interface pour SAMBA/LDAP)
utilisation de l'algorithme de cryptage de authlib.py par Christopher Arndt <chris.arndt@web.de> pour
le cryptage md5 du mot de passe

:auteurs : Bruno Boiget (Bruno.Boiget@club-internet.fr)

:version : $Revision: 1.4 $

:date : $Date: 2004/06/09 07:32:48 $

:copyright : Copyright (c) EOLE 2003

"""

import platform
if platform.system().upper() == "WINDOWS":
    from os import environ
    environ['PATH'] = environ['PATH'] + ';bin'
    if environ.has_key('PYTHONPATH'):
        environ['PYTHONPATH'] = environ['PYTHONPATH'] + ';bin'
    else:
        environ['PYTHONPATH'] = 'bin'
    from sys import path
    path.append('bin')

# librairies de gestion de l'interface graphique
from gtk import *
from gtk.glade import *
import gobject

# filtrage des warnings pour éviter les pbs sous windows (combos deprecated)
import warnings, sys
warnings.filterwarnings('ignore', message="use gtk.ComboBoxEntry")

# librairie RPC (on utilise celle de python 2.3 meme si python 2.2 est installé)
version = sys.version_info
if version[0] > 2 or (version[0] == 2 and version[1] >=3):
    import xmlrpclib
else:
    import xmlrpclib2 as xmlrpclib

# librairies système
import os
import socket
import sys

# fonction de conversion de l'unicode en UTF-8
def convert(objet):
    """Transforme les objets unicode contenus dans un objet en chaines
    """
    if type(objet) == list:
        l = []
        for item in objet:
            l.append(convert(item))
        return l
    if type(objet) == tuple:
        l = []
        for item in objet:
            l.append(convert(item))
        return l
    if type(objet) == dict:
        dico={}
        for cle in objet.keys():
            dico[cle] = convert(objet[cle])
        return dico
    if type(objet) == unicode:
        string =  objet.encode('UTF-8')
        return string
    return objet

# exceptions
class ErreurPreferences(Exception):
    pass

#######################################################################
#### Classe Frontal définissant le fonctionnement du frontal Horus ####
#######################################################################

class Frontal:

##########################
## ATTRIBUTS DE LA CLASSE
##########################

    passwd = "bidon"
    serveur = ""
    dialerte=None
    glade_file = "interface/frontend.glade"
    hide_all = True

##########################
## FONCTIONS DE LA CLASSE
##########################


###########################
## Fonctions diverses

    def get_pref(self,nom):
        "lit le fichier des préférences et retourne la valeur du paramètre"
        # ouverture du fichier
        if os.path.isfile("frontend.ini"):
            fic_prefs =open("frontend.ini","r")
            prefs=fic_prefs.read().splitlines()
            fic_prefs.close()
            for param in prefs:
                nompref,val = param.split("=")
                if nompref == nom:
                    return val
        # on n'a pas trouvé la valeur : erreur
        raise ErreurPreferences

    def set_pref(self,nom,valeur):
        "enregistre un paramètre dans le fichier des préférences"
        # ouverture et lecture du fichier
        if os.path.isfile("frontend.ini"):
            fic_prefs=open("frontend.ini","r")
            prefs=fic_prefs.read().splitlines()
            fic_prefs.close()
            # on recrée la liste des parametres avec la bonne valeur
        else:
            prefs = []
        new_prefs=[]
        existant = 0
        # on parcourt la liste des parametres
        for param in prefs:
            nompref,val = param.split("=")
            # s'il s'agit du paramètre demandé
            if nompref == nom:
                # on le met à jour avec la valeur désirée
                new_prefs.append(str(nom)+"="+str(valeur)+"\n")
                existant = 1
            else:
                # sinon, on conserve la meme valeur
                new_prefs.append(param+"\n")
        # le parametre n'existait pas encore dans le fichier
        if existant==0:
            # on l'ajoute
            new_prefs.append(str(nom)+"="+str(valeur)+"\n")
        # on réécrit le fichier
        fic_prefs=open("frontend.ini","w")
        fic_prefs.writelines(new_prefs)


    def alerte(self,msg="message d'erreur",Critical=0,parent=None):
        "Affichage d'une alerte dans une boite de dialogue"

        if msg is None:
            msg="Erreur"
        self.dialerte = XML(self.glade_file,"dialogAlerte")
        if parent is not None:
            self.dialerte.get_widget("dialogAlerte").set_transient_for(parent)
            self.dialerte.get_widget("dialogAlerte").set_position(WIN_POS_CENTER_ON_PARENT)
        self.dialerte.get_widget("labelAlerte").set_text(msg)
        # argument critical : si vrai, on ferme l'application
        #if Critical==0:
        #    self.dic_alerte = { "on_buttonAlerte_clicked" : self.CloseWindow }
        #else:
        #    self.dic_alerte = { "on_buttonAlerte_clicked" : self.DestroyFunction }
        # connexion des signaux
        self.dialerte.get_widget("dialogAlerte").show_all()
        self.dialerte.get_widget("dialogAlerte").set_keep_above(True)
        self.dialerte.get_widget("dialogAlerte").run()
        self.dialerte.get_widget("dialogAlerte").destroy()
        #self.dialerte.signal_autoconnect(self.dic_alerte)

    def rapport(self,titre,msg="rapport",parent=None):
        dial_rapport = MessageDialog(buttons=BUTTONS_CLOSE,parent=parent)
        dial_rapport.label.set_text(titre)
        text_view=TextView()
        text_view.set_editable(0)
        buffer_rapport = text_view.get_buffer()
        buffer_rapport.set_text(msg)
        sw = ScrolledWindow()
        sw.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        sw.add(text_view)
        dial_rapport.vbox.pack_start(sw,True,True,0)
        dial_rapport.set_size_request(250, 350)
        dial_rapport.show_all()
        dial_rapport.run()
        dial_rapport.destroy()


    def get_groups(self):
        return convert(self.horus.get_groups(self.serveur))


    def get_machines(self):
        return convert(self.horus.get_machines(self.serveur))


    def get_printers(self):
        return convert(self.horus.get_printersIp(self.serveur))


    def affGroupFunction(self,user):
        "récupération et affichage des groupes dans une clist de la fenêtre principale"
        try:
            liste_group = convert(self.horus.get_usergroups(user,"inscrits",self.serveur))
        except:
            result = self.alerte("Erreur LDAP lors de la lecture des groupes")
        else:
            self.affList(self.widgets,"clistGroups",liste_group)


    def affUsers(self):
        "récupération et affichage des utilisateurs dans une clist de la fenêtre principale"
        # on récupère la liste des utilisateurs
        try:
            liste=convert(self.horus.get_users(self.serveur))
        except:
            liste=[]
        self.affList(self.widgets,"clistUsers",liste)


    def affList(self,XML,clist,datalist):
        "Fonction générique d'affichage d'une liste dans une clist"
        # mise à jour de la fenêtre
        XML.get_widget(clist).clear()
        # on gèle la fenêtre pour éviter qu'elle clignote pendant la mise à jour
        XML.get_widget(clist).freeze()
        # on ajoute les groupes à la liste
        for indice in range(len(datalist)):
            if (clist == "clistPrinters"):
                #cas spécial : liste à deux colonnes
                XML.get_widget(clist).append((datalist[indice]))
            else:
                XML.get_widget(clist).append([datalist[indice]])
        # on raffraichit l'affichage de la liste
        XML.get_widget(clist).thaw()

    def CloneFunction(self,*args):
        """recopie les groupes d'un utilisateur sur un autre"""
        #demande du nom de l'utilisateur à cloner
        clone_tree=XML(self.glade_file,"dialog_clone")
        dial_clone=clone_tree.get_widget('dialog_clone')
        combo=clone_tree.get_widget('combo_clone')
        entry=clone_tree.get_widget('combo-entry_clone')
        #on récupère le nom de l'utilisateur courant
        try:
            user = self.widgets.get_widget("clistUsers").get_text(self.widgets.get_widget("clistUsers").selection[0],0)
        except:
            return
        #on met la liste des utilisateurs dans la combo-box
        liste=convert(self.horus.get_users(self.serveur))
        combo.set_popdown_strings(liste)
        #exécution du dialogue
        user_valid = 0
        while user_valid != 1:
            reponse=dial_clone.run()
            if reponse == RESPONSE_OK:
                #on regarde le nom d'utilisateur choisi et on vérifie son existence
                clone_user = entry.get_text()
                if clone_user in liste:
                    #utilisateur valide
                    user_valid=1
                else:
                    #non valide : on recommence
                    self.alerte('choisissez un utilisateur existant')
            else:
                dial_clone.destroy()
                return
        # on effectue le clonage
        # on récupère en premier la liste des groupes du user à cloner
        groupes_ori = convert(self.horus.get_usergroups(clone_user,'autres',self.serveur))
        # on récupère la liste des groupes déjà inscrits pour ne pas avoir d'erreur
        groupes_actuels = convert(self.horus.get_usergroups(user,'autres',self.serveur))
        groupes_inscr = []
        for groupe in groupes_ori:
            if groupe not in groupes_actuels:
                groupes_inscr.append(groupe)
        # on inscrit l'utilisateur à tous les groupes
        retour = convert(self.horus.inscrit(user,groupes_inscr))
        if retour != "OK":
            self.alerte(retour)
        # mise à jour de l'affichage
        self.widgets.get_widget("clistUsers").select_row(self.widgets.get_widget("clistUsers").selection[0],-1)
        dial_clone.destroy()


    def DestroyFunction(self,window,*args):
        "Fermeture de l'application"
        if self.synchro==1:
            #le serveur doit être synchronisé : message d'avertissement
            dial=MessageDialog(parent=None,type=MESSAGE_WARNING,buttons=BUTTONS_YES_NO)
            dial.label.set_text("Attention, le serveur n'est pas synchronisé, quitter ?")
            dial.show()
            reponse = dial.run()
            if reponse == RESPONSE_NO:
                dial.destroy()
                return True
        main_quit()

    def CloseWindow(self,obj=None):
        "Fermeture d'une fenêtre dont un objet est donné en paramêtre"
        test=obj.get_toplevel()
        test.destroy()

    def AboutFunction(self,*args):
        "Fenetre de dialogue d'info"
        about_dlg = MessageDialog(parent=self.widgets.get_widget("window1"),buttons=(BUTTONS_CLOSE))
        # définition du texte et de l'image à afficher
        about_dlg.label.set_text("""interface LDAP pour Horus
Eole, license GPL
Realisation par Bruno Boiget
bruno.boiget@ac-dijon.fr""")
        about_dlg.image.set_from_file("interface/images/logo.xpm")
        # mise à jour de l'affichage
        about_dlg.show_all()
        # lancement du dialogue
        about_dlg.run()
        about_dlg.destroy()


##############################
## Fonctions utilisateur

    def UserFunction(self,obj=None):
        "Initialisation de la fenêtre de gestion des utilisateurs"
        # initialisation de la fenêtre d'ajout d'utilisateur
        self.dialUser = XML(self.glade_file,"dialogUser")
        self.dialUser.get_widget("dialogUser").set_transient_for(self.widgets.get_widget("window1"))
        # on initialise la liste des groupes dans un  combobox
        self.dialUser.get_widget("comboGroups").set_popdown_strings(self.get_groups())
        # dictionnaire des signaux
        self.dic_user = { "on_buttonUserOK_clicked":self.CreateUser,
        "on_entryConf_activate":self.CreateUser,
        "on_buttonUserCancel_clicked":self.CloseWindow }
        # connexion des signaux
        self.dialUser.get_widget("checkbutton_quota").connect("toggled",lambda x: self.dialUser.get_widget("entry_quota").set_sensitive(x.get_active()))
        self.dialUser.signal_autoconnect(self.dic_user)

    def ImportUsers(self,*args):
        "Import d'une liste d'utilisateurs depuis un fichier"
        # ouverture d'une fenêtre de sélection de fichier
        fs = FileSelection(u"fichier d'utilisateurs")
        ret= fs.run()
        if ret == RESPONSE_OK:
            in_path = fs.get_filename()
            fs.destroy()
        else:
            fs.destroy()
            return

        if not os.path.isfile(in_path):
            self.alerte("le fichier "+in_path+" est invalide")

        file_users = open(in_path,"r")
        buffer_users = file_users.readlines()
        file_users.close()
        users = []
        for ligne in buffer_users:
            if not ligne.startswith("#"):
                users.append(ligne)

        # Choix des options pour les comptes
        dialImport = XML(self.glade_file,"dialogImport")
        dialImport.get_widget('checkbutton_actif').set_active(1)
        reponse = dialImport.get_widget('dialogImport').run()
        if reponse == RESPONSE_OK:
            actifs=1
            itinerants=0
            groupes_auto=0
            # on regarde l'état des cases à cocher
            if dialImport.get_widget('checkbutton_actif').get_active() == 0:
                actifs = 0
            if dialImport.get_widget('checkbutton_itinerant').get_active() == 1:
                itinerants = 1
            if dialImport.get_widget('checkbutton_groupe').get_active() == 1:
                groupes_auto = 1

            retour = convert(self.horus.import_users(users,actifs,itinerants,groupes_auto))
            dialImport.get_widget('dialogImport').destroy()
            self.affUsers()
            self.rapport("Rapport de la création",retour,dialImport.get_widget('dialogImport'))
        else:
            dialImport.get_widget('dialogImport').destroy()


    def CreateUser(self,obj=None):
        "vérification des données de l'utilisateur à créer"
        # on récupère les données saisies et on les teste
        login = self.dialUser.get_widget("entryName").get_text()
        groupe = self.dialUser.get_widget("combo-entryGroups").get_text()
        drive = self.dialUser.get_widget("entryDrive").get_text()
        password = self.dialUser.get_widget("entryPass").get_text()
        pass2 = self.dialUser.get_widget("entryConf").get_text()
        profile = self.dialUser.get_widget("checkbutton_profile").get_active()
        quota_active = self.dialUser.get_widget("checkbutton_quota").get_active()
        quota_limit = self.dialUser.get_widget("entry_quota").get_text()

        # on vérifie que toutes les données sont saisies
        if ((password == "") or
            (login == "") or
            (groupe == "") or
            ((quota_active == 1) and (quota_limit == "")) or
            (drive == "")):
            self.alerte("Certains champs ne sont pas remplis", parent=self.dialUser.get_widget('dialogUser'))
            self.dialUser.get_widget("entryName").grab_focus()
        else:
            # on vérifie la correspondance des mots de passe
            if (password != pass2):
                self.alerte("les mots de passe ne correspondent pas", parent=self.dialUser.get_widget('dialogUser'))
                self.dialUser.get_widget("entryPass").set_text("")
                self.dialUser.get_widget("entryConf").set_text("")
                self.dialUser.get_widget("entryPass").grab_focus()
            else:
                # on réalise l'ajout
                retour = convert(self.horus.add_user(login, groupe, drive, password, profile))
                if retour != "OK":
                    self.alerte(retour, parent=self.dialUser.get_widget('dialogUser'))
                elif quota_active == 1:
                    # on met en place les quotas
                    retour=convert(self.horus.set_quota_limits(login,quota_limit))
                    if retour != "OK":
                        self.alerte(retour, parent=self.dialUser.get_widget('dialogUser'))
                self.affUsers()
                obj.get_toplevel().destroy()

    def DelUserFunction(self,obj=None):
        "Demande de suppression d'un utilisateur"
        #initialisation du dialogue de suppression
        user = self.widgets.get_widget("clistUsers").get_text(self.widgets.get_widget("clistUsers").selection[0],0)
        self.dialdelUser = XML(self.glade_file,"dialogDelUser")
        self.dic_delUser = { "on_buttonConfDelUser_clicked" : self.ConfDelUserFunction,
        "on_buttonCancelDelUser_clicked":self.CloseWindow }
        self.dialdelUser.get_widget("dialogDelUser").set_title("Suppression de l'utilisateur "+user)
        self.dialdelUser.signal_autoconnect(self.dic_delUser)

    def ConfDelUserFunction(self,obj=None):
        "Suppression LDAP/Système d'un utilisateur"
        if self.dialdelUser.get_widget("checkbuttonDel").get_active():
            # on supprime aussi les répertoires et les scripts
            supflag = "systeme"
        else:
            # on supprime juste l'entrée correspondante dans l'annuaire LDAP
            supflag = "ldap"
        user = self.widgets.get_widget("clistUsers").get_text(self.widgets.get_widget("clistUsers").selection[0],0)
        # on ferme la fenêtre de dialogue
        self.CloseWindow(obj)
        # dans les deux cas, on supprime l'utilisateur
        retour = convert(self.horus.del_user(user, supflag))
        # on renvoie les éventuels messages d'erreur
        if retour != "OK":
            self.alerte(retour, parent=dialMod.get_widget("window1"))

        self.affUsers()
        self.widgets.get_widget("buttonDel").set_sensitive(0)
        self.widgets.get_widget("buttonDelegue").set_sensitive(0)
        self.widgets.get_widget("buttonInscr").set_sensitive(0)
        self.widgets.get_widget("button_clone").set_sensitive(0)
        self.widgets.get_widget("buttonDesinscr").set_sensitive(0)
        self.widgets.get_widget("buttonModif").set_sensitive(0)

    def ModUserFunction(self,obj,*args):
        """ fonction de modification d'un utilisateur (seulement le mot de passe pour l'instant)
            XXX FIXME : utiliser la fenêtre de création en grisant les
            champs non modifiables et permettre la modification de la lettre de lecteur
        """
        numUser=self.widgets.get_widget("clistUsers").selection[0]
        #initialisation du dialogue de modification
        # récupérer les données de l'utilisateur via le serveur
        # (homedrive, enable/disable, groupe)

        user = self.widgets.get_widget("clistUsers").get_text(numUser,0)

        if user == "root":
            self.alerte("Impossible de modifier root", parent=self.widgets.get_widget("window1"))
            return
        try:
            (homedrive,acct_flags,groupe,profile)=convert(self.horus.get_user_data(user,self.serveur))
        except ValueError:
            self.alerte("Erreur de lecture des informations", parent=self.widgets.get_widget("window1"))
            return

        dialMod=XML(self.glade_file,"dialogUser")

        login_entry = dialMod.get_widget("entryName")
        groupe_entry = dialMod.get_widget("combo-entryGroups")
        drive_entry = dialMod.get_widget("entryDrive")
        pwd_entry = dialMod.get_widget("entryPass")
        conf_entry = dialMod.get_widget("entryConf")
        profile_checkbox = dialMod.get_widget("checkbutton_profile")
        quota_checkbox = dialMod.get_widget("checkbutton_quota")
        quota_entry = dialMod.get_widget("entry_quota")
        quota_checkbox.connect("toggled",lambda x: quota_entry.set_sensitive(x.get_active()))

        mdpOK=0
        dialMod.get_widget("dialogUser").set_title("modification de "+str(user))
        dialMod.get_widget("dialogUser").set_transient_for(self.widgets.get_widget("window1"))
        check_enable = CheckButton("Desactiver cet utilisateur")
        dialMod.get_widget("dialog-vbox_user").pack_end(check_enable)
        check_enable.show()
        # on remplit les champs avec les anciennes valeurs
        login_entry.set_text(user)
        # on empêche la modification du nom
        login_entry.set_sensitive(0)
        drive_entry.set_text(homedrive)
        if profile == 1:
            profile_checkbox.set_active(1)
            profile_ori = True
        else:
            profile_checkbox.set_active(0)
            profile_ori = False
        # maintenant on peut changer le groupe principal
        groups = self.get_groups()
        if groupe not in groups:
            # cas particulier où le groupe principal n'est pas dans la liste
            groups.append(groupe)
        dialMod.get_widget("comboGroups").set_popdown_strings(groups)
        groupe_entry.set_text(groupe)
        #dialMod.get_widget("comboGroups").set_sensitive(0)
        # analyse des flags
        if acct_flags.count('D') >= 1:
            check_enable.set_active(1)
            enable_ori = False
        else:
            enable_ori = True
        # recherche des quotas
        quota = convert(self.horus.get_quota_limits(user))
        if quota != 0:
            quota_entry.set_text(str(quota))
            quota_checkbox.set_active(1)

        # on saisit les nouvelles données
        while (mdpOK==0):
            ret=dialMod.get_widget("dialogUser").run()
            if ret == RESPONSE_OK:
                # on récupère les différentes données à modifier
                if pwd_entry.get_text() != "":
                    # on vérifie le mot de passe
                    Pwd=pwd_entry.get_text()
                    Conf=conf_entry.get_text()
                    if ((Pwd == "") or (Conf=="")):
                        self.alerte("Tous les champs ne sont pas remplis", parent=dialMod.get_widget("dialogUser"))
                        dialMod.get_widget("entryPass").grab_focus()
                        continue
                    else:
                        if (Pwd != Conf):
                            self.alerte("Les mots de passe ne correspondent pas", parent=dialMod.get_widget("dialogUser"))
                            pwd_entry.set_text("")
                            conf_entry.set_text("")
                            pwd_entry.grab_focus()
                            continue
                else:
                    Pwd = None
                # mise en place des quotas si ils ont changé
                if not quota_checkbox.get_active():
                    new_quota = '0'
                else:
                    new_quota = quota_entry.get_text()
                try:
                    int(new_quota)
                except ValueError:
                    self.alerte("Quota utilisateur invalide", parent=dialMod.get_widget("dialogUser"))
                    quota_entry.grab_focus()
                    continue
                mdpOK = 1
            else:
                # on a demandé la fermeture du dialogue sans modifier l'utilisateur
                mdpOk=1
                dialMod.get_widget("dialogUser").destroy()
                return

        ## Toutes les modifications ##
        # lettre de lecteur
        if drive_entry.get_text() != homedrive:
            homedrive = drive_entry.get_text()
        else:
            homedrive = "None"
        # groupe principal
        if dialMod.get_widget("combo-entryGroups").get_text() != groupe:
            gid = dialMod.get_widget("combo-entryGroups").get_text()
        else:
            gid = "None"
        # utilisateur actif
        if check_enable.get_active() == enable_ori :
            enable = not check_enable.get_active()
        else:
            enable = "None"
        # profil itinérant
        if profile_checkbox.get_active() != profile_ori:
            profile = profile_checkbox.get_active()
        else:
            profile = "None"

        # on effectue la mise à jour si des données ont été changées
        if (homedrive != "None") or (gid != "None") or (enable != "None") or (profile != "None") or (Pwd is not None) :
            if Pwd is None:
                Pwd = "None"
            retour = convert(self.horus.mod_user(user, Pwd, homedrive, enable, profile, gid))
            if retour != "OK":
                # la mise à jour s'est mal passée
                self.alerte(retour, parent=dialMod.get_widget("dialogUser"))

        if str(quota) != new_quota:
            retour = convert(self.horus.set_quota_limits(user,new_quota))
            if retour != "OK":
                self.alerte(retour, parent=dialMod.get_widget("dialogUser"))

        if gid != "None":
            # mise à jour des groupes de l'utilisateur dans la fenêtre principale
            self.affGroupFunction(user)

        dialMod.get_widget("dialogUser").destroy()

    def DelegueFunction(self,obj=None):
        "Groupes administrés par un utilisateur"
        #initialisation du dialogue
        user = self.widgets.get_widget("clistUsers").get_text(self.widgets.get_widget("clistUsers").selection[0],0)
        self.dialDelegue = XML(self.glade_file,"dialogDelegue")
        self.dic_dialDelegue = { "on_ButtonDelegueOk_clicked" : self.ConfDelegueFunction,
                                 "on_treeDelegue_button_press_event":self.GetDelegueClicked,
                                 "on_ButtonDelegueCancel_clicked":self.CloseWindow }
        self.dialDelegue.get_widget("dialogDelegue").set_title(u"Groupes administrés par %s"%user)
        self.dialDelegue.signal_autoconnect(self.dic_dialDelegue)
        tree = self.dialDelegue.get_widget('treeDelegue')
        # initialisation du treeview
        store = ListStore(gobject.TYPE_STRING,    # priority
                              gobject.TYPE_BOOLEAN,   # log ?
                              )
        tree.set_model(store)
        # création des colonnes
        renderer1 = CellRendererText()
        renderer2 = CellRendererToggle()
        tree.append_column(TreeViewColumn('groupe', renderer1, text=0))
        tree.append_column(TreeViewColumn('', renderer2, active=1))

        # remplissage du modèle
        groups = self.horus.get_groups()
        admin_groups = self.horus.get_admin_groups(user)
        for g in groups:
            store.append([g,g in admin_groups])

    def GetDelegueClicked(self,treeview,event,*args):
        if event.type == gdk._2BUTTON_PRESS:
            selection = treeview.get_selection()
            store, iter_sel  = selection.get_selected()
            if iter_sel is not None:
                actif=store.get_value(iter_sel,1)
                store.set_value(iter_sel,1,not actif)

    def ConfDelegueFunction(self,obj=None):
        "Modification de la liste des groupes administrés par un utilisateur"
        # récupération des données du treeview
        user = self.widgets.get_widget("clistUsers").get_text(self.widgets.get_widget("clistUsers").selection[0],0)
        tree = self.dialDelegue.get_widget('treeDelegue')
        store = tree.get_model()
        cur_iter = store.get_iter_first()
        groups = []
        while cur_iter != None:
            gr = store.get_value(cur_iter,0)
            actif = store.get_value(cur_iter,1)
            if actif == True:
                groups.append(gr)
            cur_iter = store.iter_next(cur_iter)
        self.horus.set_admin_groups(user, groups)
        self.CloseWindow(obj)

    # Sélection des groupes dans lesquels inscrire l'utilisateur
    def SelInscrFunction(self,obj=None):
        "Affichage des groupes auxquels l'utilisateur n'appartient pas"
        # on récupère l'utilisateur selectionné
        user = self.widgets.get_widget("clistUsers").get_text(self.widgets.get_widget("clistUsers").selection[0],0)
        try:
            # on récupère la liste des groupes dans lesquels il ne figure pas
            groupes_dispo = convert(self.horus.get_usergroups(user,"dispo",self.serveur))
        except:
            self.alerte("Erreur LDAP lors de la recuperation des groupes", parent=self.widgets.get_widget("window1"))
        else:
            self.widgInscr = XML(self.glade_file,"windowInscr")
            self.dic_inscr = { "on_buttonValInscr_clicked" : self.InscrUserFunction,
            "on_buttonCancelInscr_clicked" : self.CloseWindow}
            self.widgInscr.signal_autoconnect(self.dic_inscr)
            # on affiche la liste des groupes disponibles
            self.affList(self.widgInscr,"clistInscr",groupes_dispo)


    def InscrUserFunction(self,obj=None):
        "Inscription de l'utilisateur aux groupes choisis"
        user = self.widgets.get_widget("clistUsers").get_text(self.widgets.get_widget("clistUsers").selection[0],0)
        #création de la liste des groupes choisis
        selgroup=self.widgInscr.get_widget("clistInscr").selection
        liste_ajout=[]
        for indice in range(len(selgroup)):
            liste_ajout.append(self.widgInscr.get_widget("clistInscr").get_text(selgroup[indice],0))

        # on effectue l'inscription sur le serveur
        retour = convert(self.horus.inscrit(user,liste_ajout))
        if retour != "OK":
            self.alerte(retour, parent=self.widgets.get_widget("window1"))
        else:
            # les groupes sont ajoutés, on met l'affichage de la fenêtre principale à jour
            self.affGroupFunction(user)
            self.CloseWindow(obj)


    def DesinscrFunction(self,widget,*args):
        numgroups=self.widgets.get_widget("clistGroups").selection
        #initialisation du dialogue de suppression
        groupes = []
        for numgroup in numgroups:
            groupes.append(self.widgets.get_widget("clistGroups").get_text(numgroup,0))
        if groupes != []:
            user = self.widgets.get_widget("clistUsers").get_text(self.widgets.get_widget("clistUsers").selection[0],0)
            dialDesinscr=MessageDialog(parent=self.widgets.get_widget("window1"),buttons=BUTTONS_YES_NO)
            if len(groupes) == 1:
                dialDesinscr.label.set_text("voulez vous retirer "+user+" du groupe "+groupes[0]+" ?")
            else:
                text = "voulez vous retirer "+user+" des groupe suivants ?"
                for groupe in groupes:
                    text += "\n - %s" % groupe
                dialDesinscr.label.set_text(text)

            dialDesinscr.set_default_response(RESPONSE_NO)
            ret=dialDesinscr.run()
            # on teste le retour du dialogue
            if ret == RESPONSE_YES:
                retour=convert(self.horus.desinscrit(user,groupes,self.serveur))
                if retour != "OK":
                    self.alerte(str(retour), parent=self.widgets.get_widget("window1"))
                else:
                    if not user in convert(self.horus.get_users(self.serveur)):
                        # cas particulier : perte du contrôle de l'utilisateur
                        self.affUsers()
                        self.alerte("Perte des droits d'administration sur l'utilisateur %s"%user, parent=self.widgets.get_widget("window1"))
                    else:
                        # on met à jour la liste des groupes de l'utilisateur
                        self.affGroupFunction(user)

            dialDesinscr.destroy()

    def GroupFunction(self,obj=None):
        """Fonction de gestion de la fenêtre des groupes
        """
        # création de la fenêtre de gestion des groupes
        self.dialGroup = XML(self.glade_file,"dialogGroup")
        self.dic_group = { "on_button_share_create_clicked" : self.AddShareFunction,
                           "on_button_share_delete_clicked" : self.DelShareFunction,
                           "on_button_group_create_clicked" : self.AddGroupFunction,
                           "on_button_group_delete_clicked" : self.DelGroupFunction,
                           "on_combo-entry_group_choice_changed" : self.SelectGroupFunction,
                           "on_button_goup_desinscr_clicked" : self.GroupDesinscrFunction,
                           "on_treeview_shares_button_press_event" : self.ModShareFunction
                           }

        comboGroup = self.dialGroup.get_widget("combo_group_choice")
        self.dialGroup.get_widget("dialogGroup").set_size_request(500,350)
        self.dialGroup.get_widget("dialogGroup").set_transient_for(self.widgets.get_widget('window1'))
        # gestion de la selection du partage
        # initialisation du treeview
        self.tree_shares = self.dialGroup.get_widget("treeview_shares")
        self.shares_model=ListStore(gobject.TYPE_STRING,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING)
        self.tree_shares.set_model(self.shares_model)
        column1 = TreeViewColumn("Nom de partage",CellRendererText(),text=0)
        column1.set_resizable(1)
        column2 = TreeViewColumn("Lettre",CellRendererText(),text=1)
        column2.set_resizable(1)
        column3 = TreeViewColumn("Chemin",CellRendererText(),text=2)
        column3.set_resizable(1)
        self.tree_shares.append_column(column1)
        self.tree_shares.append_column(column2)
        self.tree_shares.append_column(column3)


        # on récupère la liste des partages spéciaux (lettre de connexion forcée)
        self.special_shares=convert(self.horus.get_special_shares(self.serveur))

        comboGroup.set_popdown_strings(self.get_groups())

        # affichage de la liste des utilisateurs concernés
        self.tree_users = self.dialGroup.get_widget("treeview_groupusers")
        selection = self.tree_users.get_selection()
        selection.set_mode(SELECTION_MULTIPLE)
        self.users_model=ListStore(gobject.TYPE_STRING)
        self.tree_users.set_model(self.users_model)
        column_users = TreeViewColumn("utilisateurs",CellRendererText(),text=0)
        self.tree_users.append_column(column_users)

        self.dialGroup.signal_autoconnect(self.dic_group)
        self.SelectGroupFunction()
        self.dialGroup.get_widget("dialogGroup").run()
        self.dialGroup.get_widget("dialogGroup").destroy()

    def SelectGroupFunction(self,obj=None):
        """Fonction de mise à jour de l'affichage à la sélection d'un groupe
        """
        # on récupère le nom du groupe choisi
        groupe = self.dialGroup.get_widget("combo-entry_group_choice").get_text()
        # on recherche les partages associés
        shares = convert(self.horus.get_shares(groupe,self.serveur))
        self.shares_model.clear()
        if type(shares) != list:
            self.alerte("Erreur de recherche des partages du groupe {0}:\n\n{1}".format(groupe, shares))
        else:
            for share in shares:
                add_iter = self.shares_model.append()
                self.shares_model.set_value(add_iter,0,share[0])
                if self.special_shares.has_key(share[0]):
                    self.shares_model.set_value(add_iter,1,self.special_shares[share[0]])
                else:
                    self.shares_model.set_value(add_iter,1,"")
                self.shares_model.set_value(add_iter,2,share[1])

        # on recherche les utilisateurs associés
        users = convert(self.horus.get_groupusers(groupe,self.serveur))
        self.users_model.clear()
        if type(users) != list:
            self.alerte("Erreur de recherche des utilisateurs du groupe {0}:\n\n{1}".format(groupe, users))
        else:
            for user in users:
                add_iter = self.users_model.append()
                self.users_model.set_value(add_iter,0,user)

    def AddShareFunction(self,obj=None):
        """création d'un partage associé au groupe choisi
        """
        try:
            # modèles de partage Eole2
            models = self.horus.get_share_templates()
        except:
            models = []
        # on récupère le groupe choisi
        groupe = self.dialGroup.get_widget("combo-entry_group_choice").get_text()
        # on demande le nom du partage à ajouter
        dialShareAdd=MessageDialog(parent=self.widgets.get_widget("dialogGroup"),buttons=BUTTONS_OK_CANCEL)
        dialShareAdd.set_title("Ajout d'un partage au groupe "+groupe)
        dialShareAdd.label.set_text("Entrez le nom du nouveau partage")
        dialShareAdd.vbox.set_spacing(4)
        special_box=HBox()
        check_special = CheckButton("forcer une lettre")
        check_sticky  = CheckButton("sticky bit activé")
        #check_sticky.set_active(True)
        ComboDrive = Combo()
        lettres = ['A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z']
        ComboDrive.set_popdown_strings(lettres)
        ComboDrive.set_value_in_list(True,False)
        ComboDrive.set_use_arrows(True)
        ComboDrive.set_size_request(60,-1)
        entryDrive = ComboDrive.entry
        entryDrive.set_activates_default(True)
        entryDrive.set_sensitive(0)
        check_special.connect("toggled",lambda x: entryDrive.set_sensitive(x.get_active()))
        special_box.pack_start(check_special,False,False,0)
        special_box.pack_start(ComboDrive,True,False,0)
        entryNom=Entry()
        entryPath=Entry()
        entryNom.connect('focus_out_event',lambda x,y: entryPath.set_text("/home/workgroups/"+x.get_text()))
        labelPath=Label("chemin local")
        path_box = HBox()
        path_box.pack_start(labelPath,True,True,0)
        path_box.pack_start(entryPath,True,True,0)

        if models != []:
            labelModel = Label("modèle de partage")
            ComboModel = Combo()
            ComboModel.set_popdown_strings(models)
            model_box = HBox()
            model_box.pack_start(labelModel,True,True,0)
            model_box.pack_start(ComboModel,True,True,0)

        dialShareAdd.vbox.pack_start(entryNom,True,True,0)
        dialShareAdd.vbox.pack_start(path_box)
        if models != []:
            dialShareAdd.vbox.pack_start(model_box)
        dialShareAdd.vbox.pack_start(special_box)
        dialShareAdd.vbox.pack_start(check_sticky)

        dialShareAdd.set_default_response(RESPONSE_OK)
        entryNom.set_activates_default(RESPONSE_OK)
        dialShareAdd.show_all()
        saisie_ok = False
        while not saisie_ok:
            ret = dialShareAdd.run()
            if ret == RESPONSE_OK:
                new_share = entryNom.get_text()
                share_path = entryPath.get_text()
                drive = "None"
                if check_special.get_active() == 1:
                    drive = entryDrive.get_text()
                    if drive != "":
                        drive = drive + ":"
                if check_sticky.get_active() == 1:
                    sticky = True
                else:
                    sticky = False
                if ((new_share == "") or (share_path == "")):
                    saisie_ok=0
                    self.alerte("Veuillez remplir tous les champs")
                    entryNom.grab_focus()
                    continue
                else:
                    saisie_ok = True
                if models == []:
                    retour = convert(self.horus.add_share(new_share,groupe,self.serveur,share_path,drive,sticky))
                else:
                    model = ComboModel.entry.get_text()
                    retour = convert(self.horus.add_share(new_share,groupe,self.serveur,share_path,drive,sticky,model))

                if retour == "OK":
                    if drive != "None":
                        self.special_shares[new_share]=drive
                    # mise à jour du treeview
                    add_iter = self.shares_model.append()
                    self.shares_model.set_value(add_iter,0,new_share)
                    self.shares_model.set_value(add_iter,2,share_path)
                    if self.special_shares.has_key(new_share):
                        self.shares_model.set_value(add_iter,1,self.special_shares[new_share])
                    else:
                        self.shares_model.set_value(add_iter,1,"")
                    # on autorise à nouveau la synchronisation du serveur
                    # pour créer le répertoire
                    self.synchro=1
                    self.widgets.get_widget("buttonSynchro").set_sensitive(1)
                else:
                    self.alerte(retour, parent = dialShareAdd)
            else:
                saisie_ok = True

        dialShareAdd.destroy()

    def ModShareFunction(self,treeview,event):
        if event.type != gdk._2BUTTON_PRESS:
            return False
        tree_model,sel = treeview.get_selection().get_selected()
        if sel is not None:
            share = tree_model.get_value(sel,0)

        try:
            # modèles de partage Eole2
            models = self.horus.get_share_templates()
        except:
            models = []

        # on récupère le groupe choisi
        #groupe = self.dialGroup.get_widget("combo-entry_group_choice").get_text()
        # on demande le nom du partage à ajouter
        dialShareAdd=MessageDialog(parent=self.widgets.get_widget("dialogGroup"),buttons=BUTTONS_OK_CANCEL)
        dialShareAdd.set_title("Modification partage")
        dialShareAdd.label.set_text("Modification du partage %s"%share)
        dialShareAdd.vbox.set_spacing(4)
        special_box=HBox()
        check_special = CheckButton("forcer une lettre")
        ComboDrive = Combo()
        lettres = ['A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z']
        ComboDrive.set_popdown_strings(lettres)
        ComboDrive.set_value_in_list(True,False)
        ComboDrive.set_use_arrows(True)
        ComboDrive.set_size_request(60,-1)
        entryDrive = ComboDrive.entry
        entryDrive.set_activates_default(True)
        if share in self.special_shares.keys():
            old_drive = self.special_shares[share]
            check_special.set_active(True)
            entryDrive.set_text(old_drive[0])
        else:
            entryDrive.set_sensitive(0)
            old_drive = ""
        check_special.connect("toggled",lambda x: entryDrive.set_sensitive(x.get_active()))
        special_box.pack_start(check_special,False,False,0)
        special_box.pack_start(ComboDrive,True,False,0)
        #entryNom=Entry()
        #entryPath=Entry()
        #entryNom.connect('focus_out_event',lambda x,y: entryPath.set_text("/home/workgroups/"+x.get_text()))
        #labelPath=Label("chemin local")
        #path_box = HBox()
        #path_box.pack_start(labelPath,True,True,0)
        #path_box.pack_start(entryPath,True,True,0)

        if models != []:
            old_model = self.horus.get_share_model(share)
            labelModel = Label("modèle de partage")
            ComboModel = Combo()
            ComboModel.set_popdown_strings(models)
            ComboModel.entry.set_text(old_model)
            model_box = HBox()
            model_box.pack_start(labelModel,True,True,0)
            model_box.pack_start(ComboModel,True,True,0)
            dialShareAdd.vbox.pack_start(model_box)

        #dialShareAdd.vbox.pack_start(entryNom,True,True,0)
        #dialShareAdd.vbox.pack_start(path_box)
        dialShareAdd.vbox.pack_start(special_box)
        #dialShareAdd.vbox.pack_start(check_sticky)
        dialShareAdd.set_default_response(RESPONSE_OK)
        #entryNom.set_activates_default(RESPONSE_OK)
        dialShareAdd.show_all()
        ret = dialShareAdd.run()
        if ret == RESPONSE_OK:
            #new_share = entryNom.get_text()
            #share_path = entryPath.get_text()
            drive = ""
            if check_special.get_active() == 1:
                drive = entryDrive.get_text()
                if drive != "":
                    drive = drive + ":"
            if models == []:
                # Eole1
                try:
                    retour = convert(self.horus.mod_share(share,drive))
                except:
                    retour = "Fonction non disponible"
            else:
                # Eole2
                model = ComboModel.entry.get_text()
                try:
                    retour = convert(self.horus.mod_share(share,drive,model))
                except:
                    retour = "Fonction non disponible"


            if retour == "OK":
                if drive != old_drive:
                    if drive == "":
                        del(self.special_shares[share])
                        # mise à jour du treeview
                        tree_model.set_value(sel, 1, "")
                    else:
                        self.special_shares[share] = drive
                        # mise à jour du treeview
                        tree_model.set_value(sel, 1, drive)
            ## FIXME : synchro sur changement de modèle
            #    self.synchro=1
            #    self.widgets.get_widget("buttonSynchro").set_sensitive(1)

            else:
                self.alerte(retour, parent=dialShareAdd)

        dialShareAdd.destroy()


    def select_groupusers(self,store,iter_sel,groupe):
        """Supression des utilisateurs choisis du groupe
        """
        # on récupère le nom de l'utilisateur
        username = store.get_value(iter_sel,0)
        # on le désinscrit
        retour = convert(self.horus.desinscrit(username,groupe,self.serveur))
        if retour == "OK":
            # on retire l'utilisateur du treeview
            self.list_iters.append(iter_sel)
        else:
            self.alerte(retour, self.dialGroup.get_widget("dialogGroup"))


    def GroupDesinscrFunction(self,obj=None):
        """supression d'utilisateurs associés à un groupe
        """
        groupe = self.dialGroup.get_widget("combo-entry_group_choice").get_text()
        selection = self.tree_users.get_selection()
        # FIXME
        # BEURK!! Pas terrible, mais la fonction get_selected_rows n'est pas encore implémentée
        # pour les selections de treeview
        self.list_iters = []
        # dialogue de confirmation de la suppression
        dialDel=MessageDialog(parent=self.dialGroup.get_widget("dialogGroup"),buttons=BUTTONS_YES_NO)
        dialDel.label.set_text("desinscrire le(s) utilisateur(s) choisi(s) ?")
        dialDel.show_all()
        ret=dialDel.run()
        if ret == RESPONSE_YES:
            # suppression confirmée
            selection.selected_foreach(lambda model, path, iter_sel, sel=[]: self.select_groupusers(model,iter_sel,groupe))
            for iter_sel in self.list_iters:
                self.users_model.remove(iter_sel)
        else:
            # suppression annulée
            pass
        dialDel.destroy()

    def DelShareFunction(self,obj=None):
        """supression d'un partage associé à un groupe
        """
        selection = self.tree_shares.get_selection()
        store, iter_sel = selection.get_selected()
        if iter_sel is not None:
            # dialogue de confirmation de la suppression
            dialDel=MessageDialog(parent=self.dialGroup.get_widget("dialogGroup"),buttons=BUTTONS_YES_NO)
            dialDel.label.set_text("voulez vous supprimer le partage choisi ?")
            dialDel.show_all()
            ret=dialDel.run()
            # on teste le retour du dialogue
            if ret == RESPONSE_YES:
                # FIXME
                # suppression effective du partage (LDAP ou LDAP + systeme ?)
                store, iter_sel = selection.get_selected()
                # on récupère le nom du partage à supprimer
                sharename = store.get_value(iter_sel,0)
                retour = convert(self.horus.del_share(sharename,self.serveur))
                if retour == "OK":
                    # on supprime le partage de la liste affichée
                    if self.special_shares.has_key(sharename):
                        del(self.special_shares[sharename])
                    dialDel.destroy()
                    self.shares_model.remove(iter_sel)
                    # on autorise la synchronisation
                    self.synchro=1
                    self.widgets.get_widget("buttonSynchro").set_sensitive(1)
                else:
                    dialDel.destroy()
                    self.alerte(retour, parent=self.dialGroup.get_widget("dialogGroup"))
            else:
                dialDel.destroy()



    def AddGroupFunction(self,obj=None):
        "affichage d'un dialogue de saisie du nom de groupe à ajouter"
        # demande d'un nom de groupe à ajouter
        dialGroupAdd=MessageDialog(parent=self.widgets.get_widget("dialogGroup"),buttons=BUTTONS_OK_CANCEL)
        dialGroupAdd.set_title("Saisie d'un nouveau groupe")
        dialGroupAdd.set_transient_for(self.widgets.get_widget("dialogGroup"))
        dialGroupAdd.label.set_text("Entrez le nom du nouveau groupe")
        # ajout des widgets de saisie
        entryNom=Entry()
        checkShare=CheckButton("Ajouter un partage au groupe")
        labelCheckShare=Label("Ajouter un partage au groupe")
        labelShare=Label("nom du Partage")
        entryShare=Entry()
        checkSticky  = CheckButton("sticky bit activé")
        #checkSticky.set_active(True)
        shareBox=HBox(homogeneous=0)
        shareBox.pack_start(labelShare,True,True,0)
        shareBox.pack_start(entryShare,True,True,0)
        dialGroupAdd.vbox.pack_start(entryNom,True,True,0)
        dialGroupAdd.vbox.pack_start(checkShare,True,True,0)
        dialGroupAdd.vbox.pack_start(shareBox,True,True,0)
        dialGroupAdd.vbox.pack_start(checkSticky,True,True,0)
        dialGroupAdd.set_default_response(RESPONSE_OK)
        entryNom.set_activates_default(RESPONSE_OK)
        entryShare.set_activates_default(RESPONSE_OK)
        entryShare.set_sensitive(0)
        checkSticky.set_sensitive(0)
        checkShare.connect("toggled",lambda x: entryShare.set_sensitive(x.get_active()))
        checkShare.connect("toggled",lambda x: checkSticky.set_sensitive(x.get_active()))
        dialGroupAdd.show_all()
        saisie_ok=0
        while not saisie_ok:
            saisie_ok=1
            ret = dialGroupAdd.run()
            if ret == RESPONSE_OK:
                # Création du groupe demandé
                new_group = entryNom.get_text()
                if ((new_group == "") or ((checkShare.get_active()) and (entryShare.get_text() == ""))):
                    saisie_ok=0
                    self.alerte('Veuillez remplir tous les champs', parent=dialGroupAdd)
                    entryNom.grab_focus()
                else:
                    retour = convert(self.horus.add_group(new_group))
                    if retour != "OK":
                        self.alerte(retour, parent=dialGroupAdd)
                    else:
                        # Création du partage associé
                        if checkShare.get_active():
                            share_path="/home/workgroups/"+entryShare.get_text()
                            if checkSticky.get_active():
                                sticky = True
                            else:
                                sticky = False
                            convert(self.horus.add_share(entryShare.get_text(),new_group,self.serveur,share_path,"None",sticky))
                            # on autorise à nouveau la synchronisation du serveur
                            # pour créer le répertoire
                            self.synchro=1
                            self.widgets.get_widget("buttonSynchro").set_sensitive(1)
                        combo = self.dialGroup.get_widget('combo_group_choice')
                        combo.set_popdown_strings(self.get_groups())
                        combo.entry.set_text(new_group)
                        # on rend la combobox sensible si elle ne l'était plus
                        combo.set_sensitive(1)
        dialGroupAdd.destroy()

    def DelGroupFunction(self,obj=None):
        """ Suppression d'un groupe
        """
        # on récupère le groupe choisi
        groupe = self.dialGroup.get_widget("combo-entry_group_choice").get_text()
        # dialogue de confirmation de la suppression
        dialDel=MessageDialog(parent=self.dialGroup.get_widget("dialogGroup"),buttons=BUTTONS_YES_NO)
        dialDel.label.set_text("voulez vous supprimer le groupe choisi ?")
        dialDel.show_all()
        ret=dialDel.run()
        # on teste le retour du dialogue
        if ret == RESPONSE_YES:
            # Attention : ne supprimer le groupe que si aucun utilisateur n'y est (partages aussi ?)
            retour = convert(self.horus.del_groupe(groupe,self.serveur))
            if retour != "OK":
                dialDel.destroy()
                self.alerte(str(retour), parent=self.dialGroup.get_widget("dialogGroup"))
            else:
                # on met à jour la liste des groupes
                combo = self.dialGroup.get_widget('combo_group_choice')
                groupes = self.get_groups()
                if len(groupes) > 0:
                    combo.set_popdown_strings(groupes)
                    combo.entry.set_text(groupes[0])
                else:
                    # plus de groupes, on grise la combobox
                    combo.set_sensitive(0)
                dialDel.destroy()
        else:
            dialDel.destroy()



    def UserSelectFunction(self,obj=None,row=None,col=None,event=None):
        "Affichage de la liste des groupes auxquels appartient l'utilisateur selectionné"
        user = self.widgets.get_widget("clistUsers").get_text(self.widgets.get_widget("clistUsers").selection[0],0)
        self.affGroupFunction(user)
        # ne pas débloquer les actions interdites
        if not self.hide_all:
            self.widgets.get_widget("buttonDel").set_sensitive(1)
            self.widgets.get_widget("buttonDelegue").set_sensitive(1)
            self.widgets.get_widget("button_clone").set_sensitive(1)
        self.widgets.get_widget("buttonInscr").set_sensitive(1)
        self.widgets.get_widget("buttonDesinscr").set_sensitive(1)
        self.widgets.get_widget("buttonModif").set_sensitive(1)


    def UserUnSelectFunction(self,obj=None,row=None,col=None,event=None):
        "désactivation des boutons de suppression et d'inscription"
        self.widgets.get_widget("buttonDel").set_sensitive(0)
        self.widgets.get_widget("buttonInscr").set_sensitive(0)
        self.widgets.get_widget("buttonDesinscr").set_sensitive(0)
        self.widgets.get_widget("buttonModif").set_sensitive(0)
        self.widgets.get_widget("button_clone").set_sensitive(0)


    def SynchroFunction(self,obj=None):
        "synchronisation de la configuration sur le serveur ldap"
        # affichage d'un dialogue de confirmation
        dialSync=MessageDialog(parent=self.widgets.get_widget("window1"),buttons=BUTTONS_YES_NO)
        dialSync.label.set_text("voulez vous synchroniser le \nserveur (cela peut prendre du temps) ?")
        ret=dialSync.run()
        # on teste le retour du dialogue
        if ret == RESPONSE_YES:
            retour=convert(self.horus.synchro())
            dialSync.destroy()
            dialConf=MessageDialog(parent=self.widgets.get_widget("window1"),buttons=BUTTONS_OK)
            dialConf.label.set_text(str(retour))
            dialConf.run()
            dialConf.destroy()
            # on désactive le bouton de mise à jour du serveur
            # il sera réactivé si les changements demandés le nécessitent
            self.synchro=0
            self.widgets.get_widget("buttonSynchro").set_sensitive(0)

        else:
            dialSync.destroy()

################################
## Gestion des ordinateurs

    def MachinesFunction(self,obj=None):
        "Initialisation de la fenêtre de gestion des ordinateurs"
        if self.widgets.get_widget("togglebuttonMachines").get_active():
            # on ouvre la fenêtre de gestion des machines
            self.winmach = XML(self.glade_file,"windowMachines")
            self.winmach.get_widget("windowMachines").set_transient_for(self.widgets.get_widget('window1'))
            self.dic_mach = { "on_buttonAddMachine_clicked" : self.CreateMachineFunction,
            "on_buttonDelMachine_clicked":self.DelMachineFunction,
            "on_windowMachines_destroy":self.CloseMachinesFunction,
            "on_clistMachines_unselect_row":self.UnSelectMachinesFunction,
            "on_clistMachines_select_row":self.SelectMachinesFunction,
            "on_button_close_machines_clicked":self.CloseMachinesFunction
            }
            self.winmach.signal_autoconnect(self.dic_mach)
            # initialisation de la liste
            self.affList(self.winmach,"clistMachines",self.get_machines())
        else:
            try:
                self.CloseWindow(self.winmach.get_widget("windowMachines"))
            except:
                #la fenêtre a déjà été fermée par l'utilisateur
                pass


    def SelectMachinesFunction(self,obj=None,row=None,col=None,event=None):
        "Activation du bouton de supression"
        self.winmach.get_widget("buttonDelMachine").set_sensitive(1)


    def UnSelectMachinesFunction(self,obj=None,row=None,col=None,event=None):
        "Desactivation du bouton de supression"
        self.winmach.get_widget("buttonDelMachine").set_sensitive(0)


    def CloseMachinesFunction(self,obj=None):
        "Mise à jour du bouton d'ouverture de la fenêtre des ordinateurs dans la fenêtre principale"
        self.widgets.get_widget("togglebuttonMachines").set_active(0)


    def CreateMachineFunction(self,obj=None):
        "création d'un nouvel ordinateur"
        # saisie du nom de la machine
        dialMach=MessageDialog(parent=self.winmach.get_widget("windowMachines"),buttons=BUTTONS_OK_CANCEL)
        dialMach.label.set_text("Entrez le nom de l'ordinateur")
        entryNom=Entry()
        dialMach.vbox.pack_start(entryNom,True,True,0)
        entryNom.show()
        dialMach.set_default_response(RESPONSE_OK)
        entryNom.set_activates_default(RESPONSE_OK)
        # on exécute le dialogue
        ret = dialMach.run()
        if ret == RESPONSE_OK:
            if entryNom.get_text() != "":
                retour = convert(self.horus.add_machine(entryNom.get_text()))
                if retour == "OK":
                    self.affList(self.winmach,"clistMachines",self.get_machines())
                else:
                    self.alerte(retour, parent=dialMach)
            else:
                self.alerte("vous devez entrer un nom", parent=dialMach)
            # on récupère le nom saisi
        dialMach.destroy()

    def DelMachineFunction(self,obj=None):
        "Suppression d'un ordinateur"
        # dialogue de confirmation de la suppression
        dialDel=MessageDialog(parent=self.winmach.get_widget("windowMachines"),buttons=BUTTONS_YES_NO)
        dialDel.label.set_text("voulez vous supprimer l'ordinateur choisi ?")
        ret=dialDel.run()
        # on teste le retour du dialogue
        if ret == RESPONSE_YES:
            # on recherche l'imprimante selectionnée dans la liste
            try:
                nom_mach = self.winmach.get_widget("clistMachines").get_text(self.winmach.get_widget("clistMachines").selection[0],0)
            except:
                pass
            else:
                retour=convert(self.horus.del_machine(nom_mach+"$"))
                # Mise à jour des données
                machList=self.get_machines()
                self.affList(self.winmach,"clistMachines",machList)
        dialDel.destroy()


###############################
## Gestion des imprimantes

    def PrintersFunction(self,obj=None):
        "Initialisation de la fenêtre de gestion des ordinateurs"
        if self.widgets.get_widget("togglebuttonPrinters").get_active():
            # on ouvre la fenêtre de gestion des machines
            self.winlp = XML(self.glade_file,"windowPrinters")
            self.winlp.get_widget(self.glade_file,"windowPrinters").set_transient_for(self.widgets.get_widget('window1'))
            self.dic_lp = { "on_buttonAddPrinter_clicked" : self.CreatePrinterFunction,
            "on_buttonDelPrinter_clicked":self.DelPrinterFunction,
            "on_windowPrinters_destroy":self.ClosePrintersFunction,
            "on_clistPrinters_unselect_row":self.UnSelectLpFunction,
            "on_clistPrinters_select_row":self.SelectLpFunction,
            "on_button_close_printers_clicked":self.ClosePrintersFunction
            }
            self.winlp.signal_autoconnect(self.dic_lp)
            # initialisation des données
            lpList=self.get_printers()
            self.affList(self.winlp,"clistPrinters",lpList)
        else:
            try:
                self.CloseWindow(self.winlp.get_widget("windowPrinters"))
            except:
                #la fenêtre a déjà été fermée par l'utilisateur
                pass

    def SelectLpFunction(self,obj=None,row=None,col=None,event=None):
        "Activation du bouton de supression"
        self.winlp.get_widget("buttonDelPrinter").set_sensitive(1)


    def UnSelectLpFunction(self,obj=None,row=None,col=None,event=None):
        "Desactivation du bouton de supression"
        self.winlp.get_widget("buttonDelPrinter").set_sensitive(0)


    def ClosePrintersFunction(self,obj=None):
        "Mise à jour du bouton d'ouverture de la fenêtre des ordinateurs dans la fenêtre principale"
        self.widgets.get_widget("togglebuttonPrinters").set_active(0)


    def CreatePrinterFunction(self,obj=None):
        "Création d'une nouvelle imprimante"
        # non implémenté
        pass


    def DelPrinterFunction(self,obj=None):
        "Supression d'une imprimante"
        # dialogue de confirmation de la suppression
        dialDel=MessageDialog(parent=self.winlp.get_widget("windowPrinters"),buttons=BUTTONS_YES_NO)
        dialDel.label.set_text("voulez vous supprimer l'imprimante choisie ?")
        ret=dialDel.run()
        # on teste le retour du dialogue
        if ret == RESPONSE_YES:
            # on recherche l'imprimante selectionnée dans la liste
            try:
                nom_lp = self.winlp.get_widget("clistPrinters").get_text(self.winlp.get_widget("clistPrinters").selection[0],0)
            except:
                pass
            else:
                retour=convert(self.horus.del_printer(nom_lp))
                # Mise à jour des données
                lpList=self.get_printers()
                self.affList(self.winlp,"clistPrinters",lpList)
        dialDel.destroy()


###############################################
## Fonctions d'initialisation de l'application



    def ConnectFunction(self,obj=None):
        "Vérification du mot de passe, et connexion au serveur"
        self.serveur = self.dialcon.get_widget("entryServ").get_text()
        self.passwd = self.dialcon.get_widget("entryPwd").get_text()
        self.login = self.dialcon.get_widget("entryUser").get_text()
        # on met à jour le fichier des préférences
        try:
            self.set_pref("serveur",self.serveur)
        except:
            self.alerte("modification du fichier frontend.ini impossible", parent=self.dialcon.get_widget("dialogCnx"))
        # connexion au serveur avec le mot de passe fourni
        try:
            # on créé un proxy local pour l'objet distant
            self.horus = xmlrpclib.Server("https://%s:%s@%s:7080/" % (self.login, self.passwd, str(self.serveur)))
            profil = convert(self.horus.get_my_profil())
        except xmlrpclib.ProtocolError:
            self.alerte("erreur d'authentification", parent=self.dialcon.get_widget("dialogCnx"))
            self.dialcon.get_widget("entryUser").grab_focus()
        except:
            self.alerte('erreur de connexion au serveur', parent=self.dialcon.get_widget("dialogCnx"))
        else:
            # on est connectés et authentifiés, et on détruit la boite de dialogue de connexion au serveur
            if profil == "administrateur":
                self.hide_all = False
            if not self.hide_all:
                self.widgets.get_widget("buttonUser").show()
                self.widgets.get_widget("buttonDel").show()
                self.widgets.get_widget("buttonDelegue").show()
                self.widgets.get_widget("buttonGroupes").show()
                self.widgets.get_widget("togglebuttonMachines").show()
                self.widgets.get_widget("buttonSynchro").show()
                self.widgets.get_widget("button_clone").show()
                self.widgets.get_widget("menu_import").show()
            self.dialcon.get_widget("dialogCnx").destroy()
            self.affUsers()

    def __init__(self):
        "Démarrage de l'application"

        # définition du style par défaut
        rc_parse("interface/frontend.rc")

        # test de récupération des préférences
        try:
            self.serveur=self.get_pref("serveur")
        except ErreurPreferences:
            self.serveur="127.0.0.1"

        # On initialise les composants à partir du fichier généré par glade
        self.widgets = XML(self.glade_file,"window1")

        #On mappe les handlers des signaux
        self.dic = { "on_buttonUser_clicked":self.UserFunction,
        "on_buttonDel_clicked":self.DelUserFunction,
        "on_buttonDelegue_clicked":self.DelegueFunction,
        "on_buttonGroupes_clicked":self.GroupFunction,
        "on_clistUsers_select_row":self.UserSelectFunction,
        "on_clistUsers_unselect_row":self.UserUnSelectFunction,
        "on_buttonInscr_clicked":self.SelInscrFunction,
        "on_button_clone_clicked":self.CloneFunction,
        "on_togglebuttonMachines_toggled":self.MachinesFunction,
        #"on_togglebuttonPrinters_toggled":self.PrintersFunction,
        "on_buttonDesinscr_clicked":self.DesinscrFunction,
        "on_buttonModif_clicked":self.ModUserFunction,
        "on_menu_quit_activate" : self.DestroyFunction,
        "on_window1_delete_event" : self.DestroyFunction,
        "on_menu_import_activate" : self.ImportUsers,
        "on_about_activate" : self.AboutFunction,
        "on_buttonSynchro_clicked":self.SynchroFunction }

        #connexion des signaux
        self.synchro=0
        self.widgets.get_widget("buttonSynchro").set_sensitive(0)
        self.widgets.signal_autoconnect(self.dic)

        # Récupération du serveur et mot de passe
        self.dialcon = XML(self.glade_file,"dialogCnx")
        self.dic_cnx = { "on_buttonQuit_clicked" : self.DestroyFunction,
        "on_dialogAlerte_destroy":self.DestroyFunction,
        "on_buttonConnect_clicked":self.ConnectFunction }
        self.widgets.get_widget("clistGroups").set_selection_mode(SELECTION_MULTIPLE)

        # connexion des signaux
        self.dialcon.signal_autoconnect(self.dic_cnx)
        self.dialcon.get_widget("entryServ").set_text(self.serveur)

        # on cache tout par défaut
        self.widgets.get_widget("buttonUser").hide()
        self.widgets.get_widget("buttonDel").hide()
        self.widgets.get_widget("buttonDelegue").hide()
        self.widgets.get_widget("buttonGroupes").hide()
        self.widgets.get_widget("button_clone").hide()
        self.widgets.get_widget("togglebuttonMachines").hide()
        self.widgets.get_widget("buttonSynchro").hide()
        self.widgets.get_widget("button_clone").hide()
        self.widgets.get_widget("menu_import").hide()


def run():
    main()

if __name__ == "__main__":
    FrontHorus = Frontal()
    run()
